@@ -1,10 +1,14 @@
 from __future__ import absolute_import
 
+import re
+import logging
+
 import vertica_python.errors as errors
 
 import vertica_python.vertica.messages as messages
 from vertica_python.vertica.column import Column
 
+logger = logging.getLogger('vertica')
 
 class Cursor(object):
     def __init__(self, connection, cursor_type=None):
@@ -13,7 +17,6 @@ class Cursor(object):
         self._closed = False
         self._message = None
 
-        self.last_execution = None
         self.error = None
 
         #
@@ -33,7 +36,7 @@ class Cursor(object):
     # dbApi methods
     #
 
-    def callproc(procname, parameters=None):
+    def callproc(self, procname, parameters=None):
         raise errors.NotSupportedError('Cursor.callproc() is not implemented')
 
     def close(self):
@@ -43,35 +46,51 @@ class Cursor(object):
         if self.closed():
             raise errors.Error('Cursor is closed')
 
+        self.flush_to_query_ready()
+
         if parameters:
-            # optional requirement
+            # # optional requirement
             from psycopg2.extensions import adapt
 
             if isinstance(parameters, dict):
                 for key in parameters:
-                    v = adapt(parameters[key]).getquoted()
-                    operation = operation.replace(':' + key, v)
+                    param = parameters[key]
+                    # Make sure adapt() behaves properly
+                    if isinstance(param, unicode):
+                        v = adapt(param.encode('utf8')).getquoted()
+                    else:
+                        v = adapt(param).getquoted()
+
+                    # Using a regex with word boundary to correctly handle params with similar names
+                    # such as :s and :start
+                    match_str = u':%s\\b' % unicode(key)
+                    operation = re.sub(match_str, v.decode('utf-8'), operation, re.UNICODE)
             elif isinstance(parameters, tuple):
-                operation = operation % tuple(adapt(p).getquoted() for p in parameters)
+                tlist = []
+                for p in parameters:
+                    if isinstance(p, unicode):
+                        tlist.append(adapt(p.encode('utf8')).getquoted())
+                    else:
+                        tlist.append(adapt(p).getquoted())
+                operation = operation % tuple(tlist)
             else:
                 raise errors.Error("Argument 'parameters' must be dict or tuple")
 
         self.rowcount = 0
-        if self.last_execution:
-            # ToDo: can just empty the message buffer in connection if easy to do
-            self.connection.reset_connection()
-        self.last_execution = operation
+
         self.connection.write(messages.Query(operation))
 
+        # read messages until we hit an Error, DataRow or ReadyForQuery
         while True:
             message = self.connection.read_message()
+            # save the message because there's no way to undo the read
+            self._message = message
             if isinstance(message, messages.ErrorResponse):
-                raise errors.QueryError.from_error_response(message, self.last_execution)
+                raise errors.QueryError.from_error_response(message, operation)
             elif isinstance(message, messages.RowDescription):
                 self.description = map(lambda fd: Column(fd), message.fields)
-            elif isinstance(message, messages.DataRow) \
-                    or isinstance(message, messages.ReadyForQuery):
-                self._message = message  # cache the message because there's no way to undo the read
+            elif (isinstance(message, messages.DataRow)
+                   or isinstance(message, messages.ReadyForQuery)):
                 break
             else:
                 self.connection.process_message(message)
@@ -80,6 +99,7 @@ class Cursor(object):
         if isinstance(self._message, messages.DataRow):
             self.rowcount += 1
             row = self.row_formatter(self._message)
+            # fetch next message
             self._message = self.connection.read_message()
             return row
         else:
@@ -117,6 +137,19 @@ class Cursor(object):
     # Non dbApi methods
     #
     # todo: input stream
+    def flush_to_query_ready(self):
+        # if the last message isnt empty or ReadyForQuery, read all remaining messages
+        if(self._message is None
+           or isinstance(self._message, messages.ReadyForQuery)):
+            return
+
+        while True:
+            message = self.connection.read_message()
+            if isinstance(message, messages.ReadyForQuery):
+                self.connection.transaction_status = message.transaction_status
+                self._message = message
+                break
+
     def copy(self, sql, data):
         # Legacy support
         self.copy_string(sql, data)
@@ -130,7 +163,7 @@ class Cursor(object):
         while True:
             message = self.connection.read_message()
             if isinstance(message, messages.ErrorResponse):
-                raise errors.QueryError.from_error_response(message, self.last_execution)
+                raise errors.QueryError.from_error_response(message, sql)
             elif isinstance(message, messages.ReadyForQuery):
                 break
             elif isinstance(message, messages.CopyInResponse):
@@ -148,7 +181,6 @@ class Cursor(object):
         else:
             self._copy_internal(sql, (line.decode(decoder) for line in data))
 
-
     #
     # Internal
     #
@@ -173,4 +205,4 @@ class Cursor(object):
 
     def format_row_as_array(self, row_data):
         return [self.description[idx].convert(value)
-                for idx, value in enumerate(row_data.values)]
\ No newline at end of file
+                for idx, value in enumerate(row_data.values)]
